@@ -1,16 +1,32 @@
-# -*- coding: utf-8 -*-
-import sys
 import os
 import re
-import argparse
-import string
+import sys
 import random
-from waybackpy.wrapper import Url
-from waybackpy.__version__ import __version__
+import string
+import argparse
+from .wrapper import Url
+from .exceptions import WaybackError
+from .__version__ import __version__
 
 
 def _save(obj):
-    return obj.save()
+    try:
+        return obj.save()
+    except Exception as err:
+        e = str(err)
+        m = re.search(r"Header:\n(.*)", e)
+        if m:
+            header = m.group(1)
+        if "No archive URL found in the API response" in e:
+            return (
+                "\n[waybackpy] Can not save/archive your link.\n[waybackpy] This "
+                "could happen because either your waybackpy (%s) is likely out of "
+                "date or Wayback Machine is malfunctioning.\n[waybackpy] Visit "
+                "https://github.com/akamhy/waybackpy for the latest version of "
+                "waybackpy.\n[waybackpy] API response Header :\n%s"
+                % (__version__, header)
+            )
+        return WaybackError(err)
 
 
 def _archive_url(obj):
@@ -21,12 +37,33 @@ def _json(obj):
     return obj.JSON
 
 
+def no_archive_handler(e, obj):
+    m = re.search(r"archive\sfor\s\'(.*?)\'\stry", str(e))
+    if m:
+        url = m.group(1)
+        ua = obj.user_agent
+        if "github.com/akamhy/waybackpy" in ua:
+            ua = "YOUR_USER_AGENT_HERE"
+        return (
+            "\n[Waybackpy] Can not find archive for '%s'.\n[Waybackpy] You can"
+            " save the URL using the following command:\n[Waybackpy] waybackpy --"
+            'user_agent "%s" --url "%s" --save' % (url, ua, url)
+        )
+    return WaybackError(e)
+
+
 def _oldest(obj):
-    return obj.oldest()
+    try:
+        return obj.oldest()
+    except Exception as e:
+        return no_archive_handler(e, obj)
 
 
 def _newest(obj):
-    return obj.newest()
+    try:
+        return obj.newest()
+    except Exception as e:
+        return no_archive_handler(e, obj)
 
 
 def _total_archives(obj):
@@ -35,25 +72,25 @@ def _total_archives(obj):
 
 def _near(obj, args):
     _near_args = {}
-    if args.year:
-        _near_args["year"] = args.year
-    if args.month:
-        _near_args["month"] = args.month
-    if args.day:
-        _near_args["day"] = args.day
-    if args.hour:
-        _near_args["hour"] = args.hour
-    if args.minute:
-        _near_args["minute"] = args.minute
-    return obj.near(**_near_args)
+    args_arr = [args.year, args.month, args.day, args.hour, args.minute]
+    keys = ["year", "month", "day", "hour", "minute"]
+
+    for key, arg in zip(keys, args_arr):
+        if arg:
+            _near_args[key] = arg
+
+    try:
+        return obj.near(**_near_args)
+    except Exception as e:
+        return no_archive_handler(e, obj)
 
 
 def _save_urls_on_file(input_list, live_url_count):
     m = re.search("https?://([A-Za-z_0-9.-]+).*", input_list[0])
+
+    domain = "domain-unknown"
     if m:
         domain = m.group(1)
-    else:
-        domain = "domain-unknown"
 
     uid = "".join(
         random.choice(string.ascii_lowercase + string.digits) for _ in range(6)
@@ -68,52 +105,44 @@ def _save_urls_on_file(input_list, live_url_count):
 
 
 def _known_urls(obj, args):
-    """Abbreviations:
-    sd = subdomain
-    al = alive
     """
-    sd = False
-    al = False
+    Known urls for a domain.
+    """
+
+    subdomain = False
     if args.subdomain:
-        sd = True
+        subdomain = True
+
+    alive = False
     if args.alive:
-        al = True
-    url_list = obj.known_urls(alive=al, subdomain=sd)
+        alive = True
+
+    url_list = obj.known_urls(alive=alive, subdomain=subdomain)
     total_urls = len(url_list)
 
     if total_urls > 0:
-        text = _save_urls_on_file(url_list, total_urls)
-    else:
-        text = "No known URLs found. Please try a diffrent domain!"
+        return _save_urls_on_file(url_list, total_urls)
 
-    return text
+    return "No known URLs found. Please try a diffrent domain!"
 
 
 def _get(obj, args):
     if args.get.lower() == "url":
-        output = obj.get()
-
-    elif args.get.lower() == "archive_url":
-        output = obj.get(obj.archive_url)
-
-    elif args.get.lower() == "oldest":
-        output = obj.get(obj.oldest())
-
-    elif args.get.lower() == "latest" or args.get.lower() == "newest":
-        output = obj.get(obj.newest())
-
-    elif args.get.lower() == "save":
-        output = obj.get(obj.save())
-
-    else:
-        output = "Use get as \"--get 'source'\", 'source' can be one of the followings: \
-            \n1) url - get the source code of the url specified using --url/-u.\
-            \n2) archive_url - get the source code of the newest archive for the supplied url, alias of newest.\
-            \n3) oldest - get the source code of the oldest archive for the supplied url.\
-            \n4) newest - get the source code of the newest archive for the supplied url.\
-            \n5) save - Create a new archive and get the source code of this new archive for the supplied url."
-
-    return output
+        return obj.get()
+    if args.get.lower() == "archive_url":
+        return obj.get(obj.archive_url)
+    if args.get.lower() == "oldest":
+        return obj.get(obj.oldest())
+    if args.get.lower() == "latest" or args.get.lower() == "newest":
+        return obj.get(obj.newest())
+    if args.get.lower() == "save":
+        return obj.get(obj.save())
+    return "Use get as \"--get 'source'\", 'source' can be one of the followings: \
+        \n1) url - get the source code of the url specified using --url/-u.\
+        \n2) archive_url - get the source code of the newest archive for the supplied url, alias of newest.\
+        \n3) oldest - get the source code of the oldest archive for the supplied url.\
+        \n4) newest - get the source code of the newest archive for the supplied url.\
+        \n5) save - Create a new archive and get the source code of this new archive for the supplied url."
 
 
 def args_handler(args):
@@ -145,7 +174,7 @@ def args_handler(args):
     elif args.total:
         output = _total_archives(obj)
     elif args.near:
-        output = _near(obj, args)
+        return _near(obj, args)
     elif args.get:
         output = _get(obj, args)
     else:
@@ -156,24 +185,24 @@ def args_handler(args):
     return output
 
 
-def parse_args(argv):
-    parser = argparse.ArgumentParser()
-
-    requiredArgs = parser.add_argument_group("URL argument (required)")
+def add_requiredArgs(requiredArgs):
     requiredArgs.add_argument(
         "--url", "-u", help="URL on which Wayback machine operations would occur"
     )
 
-    userAgentArg = parser.add_argument_group("User Agent")
+
+def add_userAgentArg(userAgentArg):
     help_text = 'User agent, default user_agent is "waybackpy python package - https://github.com/akamhy/waybackpy"'
     userAgentArg.add_argument("--user_agent", "-ua", help=help_text)
 
-    saveArg = parser.add_argument_group("Create new archive/save URL")
+
+def add_saveArg(saveArg):
     saveArg.add_argument(
         "--save", "-s", action="store_true", help="Save the URL on the Wayback machine"
     )
 
-    auArg = parser.add_argument_group("Get the latest Archive")
+
+def add_auArg(auArg):
     auArg.add_argument(
         "--archive_url",
         "-au",
@@ -181,7 +210,8 @@ def parse_args(argv):
         help="Get the latest archive URL, alias for --newest",
     )
 
-    jsonArg = parser.add_argument_group("Get the JSON data")
+
+def add_jsonArg(jsonArg):
     jsonArg.add_argument(
         "--json",
         "-j",
@@ -189,7 +219,8 @@ def parse_args(argv):
         help="JSON data of the availability API request",
     )
 
-    oldestArg = parser.add_argument_group("Oldest archive")
+
+def add_oldestArg(oldestArg):
     oldestArg.add_argument(
         "--oldest",
         "-o",
@@ -197,7 +228,8 @@ def parse_args(argv):
         help="Oldest archive for the specified URL",
     )
 
-    newestArg = parser.add_argument_group("Newest archive")
+
+def add_newestArg(newestArg):
     newestArg.add_argument(
         "--newest",
         "-n",
@@ -205,7 +237,8 @@ def parse_args(argv):
         help="Newest archive for the specified URL",
     )
 
-    totalArg = parser.add_argument_group("Total number of archives")
+
+def add_totalArg(totalArg):
     totalArg.add_argument(
         "--total",
         "-t",
@@ -213,16 +246,16 @@ def parse_args(argv):
         help="Total number of archives for the specified URL",
     )
 
-    getArg = parser.add_argument_group("Get source code")
+
+def add_getArg(getArg):
     getArg.add_argument(
         "--get",
         "-g",
         help="Prints the source code of the supplied url. Use '--get help' for extended usage",
     )
 
-    knownUrlArg = parser.add_argument_group(
-        "URLs known and archived to Waybcak Machine for the site."
-    )
+
+def add_knownUrlArg(knownUrlArg):
     knownUrlArg.add_argument(
         "--known_urls", "-ku", action="store_true", help="URLs known for the domain."
     )
@@ -231,31 +264,48 @@ def parse_args(argv):
     help_text = "Only include live URLs. Will not inlclude dead links."
     knownUrlArg.add_argument("--alive", "-a", action="store_true", help=help_text)
 
-    nearArg = parser.add_argument_group("Archive close to time specified")
+
+def add_nearArg(nearArg):
     nearArg.add_argument(
         "--near", "-N", action="store_true", help="Archive near specified time"
     )
 
-    nearArgs = parser.add_argument_group("Arguments that are used only with --near")
+
+def add_nearArgs(nearArgs):
     nearArgs.add_argument("--year", "-Y", type=int, help="Year in integer")
     nearArgs.add_argument("--month", "-M", type=int, help="Month in integer")
     nearArgs.add_argument("--day", "-D", type=int, help="Day in integer.")
     nearArgs.add_argument("--hour", "-H", type=int, help="Hour in intege")
     nearArgs.add_argument("--minute", "-MIN", type=int, help="Minute in integer")
 
+
+def parse_args(argv):
+    parser = argparse.ArgumentParser()
+    add_requiredArgs(parser.add_argument_group("URL argument (required)"))
+    add_userAgentArg(parser.add_argument_group("User Agent"))
+    add_saveArg(parser.add_argument_group("Create new archive/save URL"))
+    add_auArg(parser.add_argument_group("Get the latest Archive"))
+    add_jsonArg(parser.add_argument_group("Get the JSON data"))
+    add_oldestArg(parser.add_argument_group("Oldest archive"))
+    add_newestArg(parser.add_argument_group("Newest archive"))
+    add_totalArg(parser.add_argument_group("Total number of archives"))
+    add_getArg(parser.add_argument_group("Get source code"))
+    add_knownUrlArg(
+        parser.add_argument_group(
+            "URLs known and archived to Waybcak Machine for the site."
+        )
+    )
+    add_nearArg(parser.add_argument_group("Archive close to time specified"))
+    add_nearArgs(parser.add_argument_group("Arguments that are used only with --near"))
     parser.add_argument(
         "--version", "-v", action="store_true", help="Waybackpy version"
     )
-
     return parser.parse_args(argv[1:])
 
 
 def main(argv=None):
-    if argv is None:
-        argv = sys.argv
-    args = parse_args(argv)
-    output = args_handler(args)
-    print(output)
+    argv = sys.argv if argv is None else argv
+    print(args_handler(parse_args(argv)))
 
 
 if __name__ == "__main__":
