@@ -11,6 +11,7 @@ from .utils import (
     _url_check,
     _cleaned_url,
     _ts,
+    _unix_ts_to_wayback_ts,
 )
 
 
@@ -143,12 +144,17 @@ class Url:
 
     def get(self, url="", user_agent="", encoding=""):
         """
-        Return the source code of the supplied URL.
+        Return the source code of the last archived URL,
+        if no URL is passed to this method.
+
         If encoding is not supplied, it is auto-detected
          from the response itself by requests package.
         """
 
-        if not url:
+        if not url and self._archive_url:
+            url = self._archive_url
+
+        elif not url and not self._archive_url:
             url = _cleaned_url(self.url)
 
         if not user_agent:
@@ -165,7 +171,15 @@ class Url:
 
         return response.content.decode(encoding.replace("text/html", "UTF-8", 1))
 
-    def near(self, year=None, month=None, day=None, hour=None, minute=None):
+    def near(
+        self,
+        year=None,
+        month=None,
+        day=None,
+        hour=None,
+        minute=None,
+        unix_timestamp=None,
+    ):
         """
         Wayback Machine can have many archives of a webpage,
         sometimes we want archive close to a specific time.
@@ -187,14 +201,18 @@ class Url:
 
         And finally return self.
         """
-        now = datetime.utcnow().timetuple()
-        timestamp = _wayback_timestamp(
-            year=year if year else now.tm_year,
-            month=month if month else now.tm_mon,
-            day=day if day else now.tm_mday,
-            hour=hour if hour else now.tm_hour,
-            minute=minute if minute else now.tm_min,
-        )
+
+        if unix_timestamp:
+            timestamp = _unix_ts_to_wayback_ts(unix_timestamp)
+        else:
+            now = datetime.utcnow().timetuple()
+            timestamp = _wayback_timestamp(
+                year=year if year else now.tm_year,
+                month=month if month else now.tm_mon,
+                day=day if day else now.tm_mday,
+                hour=hour if hour else now.tm_hour,
+                minute=minute if minute else now.tm_min,
+            )
 
         endpoint = "https://archive.org/wayback/available"
         headers = {"User-Agent": self.user_agent}
@@ -298,9 +316,23 @@ class Url:
         url_list = []
 
         if subdomain:
-            cdx = Cdx(_cleaned_url(self.url), user_agent=self.user_agent, start_timestamp=start_timestamp, end_timestamp=end_timestamp, match_type="domain", collapses=["urlkey"])
+            cdx = Cdx(
+                _cleaned_url(self.url),
+                user_agent=self.user_agent,
+                start_timestamp=start_timestamp,
+                end_timestamp=end_timestamp,
+                match_type="domain",
+                collapses=["urlkey"],
+            )
         else:
-            cdx = Cdx(_cleaned_url(self.url), user_agent=self.user_agent, start_timestamp=start_timestamp, end_timestamp=end_timestamp, match_type="host", collapses=["urlkey"])
+            cdx = Cdx(
+                _cleaned_url(self.url),
+                user_agent=self.user_agent,
+                start_timestamp=start_timestamp,
+                end_timestamp=end_timestamp,
+                match_type="host",
+                collapses=["urlkey"],
+            )
 
         snapshots = cdx.snapshots()
 
